@@ -61,8 +61,6 @@
 
 #define sure_string(s) ((s)!=NULL?(s):"")
 
-extern char **environ;
-
 struct _GnomeDesktopItem {
 	int refcount;
 
@@ -119,7 +117,7 @@ typedef struct {
 	gboolean buf_needs_free;
 	gboolean past_first_read;
 	gboolean eof;
-	gsize size;
+	guint64 size;
 	gsize pos;
 } ReadBuf;
 
@@ -1079,6 +1077,13 @@ set_locale (GnomeDesktopItem *item, const char *key,
 		char *full = g_strdup_printf ("%s[%s]", key, locale);
 		set (item, full, value);
 		g_free (full);
+
+		/* add the locale to the list of languages if it wasn't there
+		 * before */
+		if (g_list_find_custom (item->languages, locale,
+					(GCompareFunc)strcmp) == NULL)
+			item->languages = g_list_prepend (item->languages,
+							  g_strdup (locale));
 	}
 }
 
@@ -1418,7 +1423,6 @@ do_percent_subst (const GnomeDesktopItem  *item,
 		if (g_ascii_isdigit (arg [1])) 
 			g_string_append_c (str, '%');
 		return FALSE;
-		break;
 	}
 
 	return TRUE;
@@ -1492,23 +1496,32 @@ static char **
 make_spawn_environment_for_sn_context (SnLauncherContext *sn_context,
 				       char             **envp)
 {
-	char **retval = NULL;
+	char **retval;
+	char **freeme;
 	int    i, j;
 	int    desktop_startup_id_len;
 
-	if (envp == NULL)
-		envp = environ;
-	
-	for (i = 0; envp[i]; i++)
-		;
+	retval = freeme = NULL;
+
+	if (envp == NULL) {
+		envp = freeme = g_listenv ();
+		for (i = 0; envp[i]; i++) {
+			char *name = envp[i];
+
+			envp[i] = g_strjoin ("=", name, g_getenv (name), NULL);
+			g_free (name);
+		}
+	} else {
+		for (i = 0; envp[i]; i++)
+			;
+	}
 
 	retval = g_new (char *, i + 2);
 
 	desktop_startup_id_len = strlen ("DESKTOP_STARTUP_ID");
 	
 	for (i = 0, j = 0; envp[i]; i++) {
-		if (strncmp (envp[i], "DESKTOP_STARTUP_ID", desktop_startup_id_len) != 0)
-		{
+		if (strncmp (envp[i], "DESKTOP_STARTUP_ID", desktop_startup_id_len) != 0) {
 			retval[j] = g_strdup (envp[i]);
 			++j;
 	        }
@@ -1519,6 +1532,8 @@ make_spawn_environment_for_sn_context (SnLauncherContext *sn_context,
 	++j;
 	retval[j] = NULL;
 
+	g_strfreev (freeme);
+
 	return retval;
 }
 
@@ -1666,15 +1681,25 @@ static char **
 make_environment_for_screen (GdkScreen  *screen,
 			     char      **envp)
 {
-	char **retval = NULL;
+	char **retval;
+	char **freeme;
 	char  *display_name;
 	int    display_index = -1;
 	int    i, env_len;
 
 	g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
 
-	if (envp == NULL)
-		envp = environ;
+	retval = freeme = NULL;
+
+	if (envp == NULL) {
+		envp = freeme = g_listenv ();
+		for (i = 0; envp [i]; i++) {
+			char *name = envp[i];
+
+			envp[i] = g_strjoin ("=", name, g_getenv (name), NULL);
+			g_free (name);
+		}
+	}
 
 	for (env_len = 0; envp [env_len]; env_len++)
 		if (strncmp (envp [env_len], "DISPLAY", strlen ("DISPLAY")) == 0)
@@ -1694,6 +1719,7 @@ make_environment_for_screen (GdkScreen  *screen,
 	g_assert (i == env_len);
 
 	g_free (display_name);
+	g_free (freeme);
 
 	return retval;
 }
@@ -1733,7 +1759,14 @@ ditem_execute (const GnomeDesktopItem *item,
 	
 	g_return_val_if_fail (item, -1);
 
-	if (!use_current_dir)
+	if (item->type == GNOME_DESKTOP_ITEM_TYPE_APPLICATION) {
+		working_dir = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_PATH);
+		if (working_dir &&
+		    !g_file_test (working_dir, G_FILE_TEST_IS_DIR))
+			working_dir = NULL;
+	}
+
+	if (working_dir == NULL && !use_current_dir)
 		working_dir = g_get_home_dir ();
 
 	if (gnome_desktop_item_get_boolean (item, GNOME_DESKTOP_ITEM_TERMINAL)) {
@@ -1896,7 +1929,7 @@ ditem_execute (const GnomeDesktopItem *item,
 			if (item->launch_time > 0)
 				launch_time = item->launch_time;
 			else
-				launch_time = gtk_get_current_event_time ();
+				launch_time = gdk_x11_display_get_user_time (gdk_screen_get_display (screen));
 
 			sn_launcher_context_initiate (sn_context,
 						      g_get_prgname () ? g_get_prgname () : "unknown",
@@ -2011,25 +2044,26 @@ gnome_desktop_item_launch_on_screen_with_env (
 	/* This is a URL, so launch it as a url */
 	if (item->type == GNOME_DESKTOP_ITEM_TYPE_LINK) {
 		const char *url;
+		char       *free_url;
+		gboolean    retval;
+
 		url = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_URL);
-		if (url && url[0] != '\0') {
-			if (gnome_url_show (url, error))
-				return 0;
-			else
-				return -1;
+		if (url && url[0] != '\0')
+			free_url = gnome_vfs_make_uri_canonical (url);
 		/* Gnome panel used to put this in Exec */
-		} else if (exec && exec[0] != '\0') {
-			if (gnome_url_show (exec, error))
-				return 0;
-			else
-				return -1;
-		} else {
+		else if (exec && exec[0] != '\0')
+			free_url = gnome_vfs_make_uri_canonical (exec);
+		else {
 			g_set_error (error,
 				     GNOME_DESKTOP_ITEM_ERROR,
 				     GNOME_DESKTOP_ITEM_ERROR_NO_URL,
 				     _("No URL to launch"));
 			return -1;
 		}
+
+		retval = gnome_url_show (free_url, error);
+		g_free (free_url);
+		return retval ? 0 : -1;
 	}
 
 	/* check the type, if there is one set */
@@ -2592,6 +2626,7 @@ find_kde_directory (void)
 
 /**
  * gnome_desktop_item_find_icon:
+ * @icon_theme: a #GtkIconTheme
  * @icon: icon name, something you'd get out of the Icon key
  * @desired_size: FIXME
  * @flags: FIXME
@@ -2603,13 +2638,17 @@ find_kde_directory (void)
  * Returns: A newly allocated string
  */
 char *
-gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
+gnome_desktop_item_find_icon (GtkIconTheme *icon_theme,
 			      const char *icon,
 			      int desired_size,
 			      int flags)
 {
 	char *full = NULL;
 
+	g_return_val_if_fail (icon_theme == NULL ||
+			      GTK_IS_ICON_THEME (icon_theme) ||
+			      GNOME_IS_ICON_THEME (icon_theme), NULL);
+
 	if (icon == NULL || strcmp(icon,"") == 0) {
 		return NULL;
 	} else if (g_path_is_absolute (icon)) {
@@ -2623,11 +2662,10 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
 		char *p;
 
 		if (icon_theme == NULL) {
-			icon_theme = gnome_icon_theme_new ();
+			icon_theme = gtk_icon_theme_new ();
 		} else {
 			g_object_ref (icon_theme);
 		}
-
 		
 		icon_no_extension = g_strdup (icon);
 		p = strrchr (icon_no_extension, '.');
@@ -2638,10 +2676,26 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
 		    *p = 0;
 		}
 
-		full = gnome_icon_theme_lookup_icon (icon_theme,
-						     icon_no_extension,
-						     desired_size,
-						     NULL, NULL);
+		/* For backwards compat we support GnomeIconTheme too */
+		if (GNOME_IS_ICON_THEME (icon_theme)) {
+			full = gnome_icon_theme_lookup_icon (icon_theme,
+							     icon_no_extension,
+							     desired_size,
+							     NULL, NULL);
+		} else {
+			GtkIconInfo *info;
+
+			info = gtk_icon_theme_lookup_icon (icon_theme,
+							   icon_no_extension,
+							   desired_size,
+							   0);
+
+			full = NULL;
+			if (info) {
+				full = g_strdup (gtk_icon_info_get_filename (info));
+				gtk_icon_info_free (info);
+			}
+		}
 		
 		g_object_unref (icon_theme);
 		
@@ -2704,6 +2758,7 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
 
 /**
  * gnome_desktop_item_get_icon:
+ * @icon_theme: a #GtkIconTheme
  * @item: A desktop item
  *
  * Description:  This function goes and looks for the icon file.  If the icon
@@ -2714,7 +2769,7 @@ gnome_desktop_item_find_icon (GnomeIconTheme *icon_theme,
  */
 char *
 gnome_desktop_item_get_icon (const GnomeDesktopItem *item,
-			     GnomeIconTheme *icon_theme)
+			     GtkIconTheme *icon_theme)
 {
 	/* maybe this function should be deprecated in favour of find icon
 	 * -George */
@@ -2752,8 +2807,6 @@ gnome_desktop_item_get_location (const GnomeDesktopItem *item)
  * @location: A uri string specifying the file location of this particular item.
  *
  * Set's the 'location' uri of this item.
- *
- * Returns:
  */
 void
 gnome_desktop_item_set_location (GnomeDesktopItem *item, const char *location)
@@ -2800,8 +2853,6 @@ gnome_desktop_item_set_location (GnomeDesktopItem *item, const char *location)
  * @file: A local filename specifying the file location of this particular item.
  *
  * Set's the 'location' uri of this item to the given @file.
- *
- * Returns:
  */
 void
 gnome_desktop_item_set_location_file (GnomeDesktopItem *item, const char *file)
